//  Copyright (c) 2015 Couchbase, Inc.
//  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file
//  except in compliance with the License. You may obtain a copy of the License at
//    http://www.apache.org/licenses/LICENSE-2.0
//  Unless required by applicable law or agreed to in writing, software distributed under the
//  License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
//  either express or implied. See the License for the specific language governing permissions
//  and limitations under the License.

package ansi

import (
	"fmt"

	"git.autistici.org/ale/liber/Godeps/_workspace/src/github.com/blevesearch/bleve/registry"
	"git.autistici.org/ale/liber/Godeps/_workspace/src/github.com/blevesearch/bleve/search/highlight"
	ansi_formatter "git.autistici.org/ale/liber/Godeps/_workspace/src/github.com/blevesearch/bleve/search/highlight/fragment_formatters/ansi"
	simple_fragmenter "git.autistici.org/ale/liber/Godeps/_workspace/src/github.com/blevesearch/bleve/search/highlight/fragmenters/simple"
	simple_highlighter "git.autistici.org/ale/liber/Godeps/_workspace/src/github.com/blevesearch/bleve/search/highlight/highlighters/simple"
)

const Name = "ansi"

func Constructor(config map[string]interface{}, cache *registry.Cache) (highlight.Highlighter, error) {

	fragmenter, err := cache.FragmenterNamed(simple_fragmenter.Name)
	if err != nil {
		return nil, fmt.Errorf("error building fragmenter: %v", err)
	}

	formatter, err := cache.FragmentFormatterNamed(ansi_formatter.Name)
	if err != nil {
		return nil, fmt.Errorf("error building fragment formatter: %v", err)
	}

	return simple_highlighter.NewHighlighter(
			fragmenter,
			formatter,
			simple_highlighter.DefaultSeparator),
		nil
}

func init() {
	registry.RegisterHighlighter(Name, Constructor)
}
